@@ -37,7 +37,7 @@ class _DatasetFeatureOperand:
     feature: str
 
 
-class Formula:
+class _Formula:
     def __init__(
         self,
         name: Optional[str] = None,
@@ -48,24 +48,24 @@ class Formula:
         self._kind = MetricFormulaKind(kind.upper()) if kind else None
         self._operands = operands
 
-    def with_name(self, name: str) -> "Formula":
+    def with_name(self, name: str) -> "_Formula":
         copy = self._copy_with()
         copy._name = name
         return copy
 
-    def with_kind(self, kind: Union[MetricFormulaKind, str]) -> "Formula":
+    def with_kind(self, kind: Union[MetricFormulaKind, str]) -> "_Formula":
         copy = self._copy_with()
         copy._kind = MetricFormulaKind(kind.upper())
         return copy
 
     def with_operands(
         self, operands: Union[_SingleSeriesOperand, _MultiSeriesOperand, _DatasetFeatureOperand]
-    ) -> "Formula":
+    ) -> "_Formula":
         copy = self._copy_with()
         copy._operands = operands
         return copy
 
-    def _copy_with(self) -> "Formula":
+    def _copy_with(self) -> "_Formula":
         self_copy = deepcopy(self)
         return self_copy
 
@@ -221,7 +221,7 @@ class Chart:
         self._name = name
         self._window_period = window_period
         self._series: List[SeriesBase] = []
-        self._formulas: List[Formula] = []
+        self._formulas: List[_Formula] = []
         self._trigger = None
         self._keep = keep
         self._entity_id = None
@@ -474,7 +474,7 @@ class Chart:
         raise ValueError(f"No series named '{series_name}' exists in Chart '{self._name}'")
 
     @overload
-    def with_formula(self, /, formula: Formula, **kwargs) -> "Chart":
+    def with_formula(self, /, formula: _Formula, **kwargs) -> "Chart":
         ...
 
     @overload
@@ -482,16 +482,16 @@ class Chart:
         ...
 
     @_copy_with
-    def with_formula(self, /, name: Optional[str] = None, formula: Optional[Formula] = None, **kwargs) -> "Chart":
+    def with_formula(self, /, name: Optional[str] = None, formula: Optional[_Formula] = None, **kwargs) -> "Chart":
         if formula:
-            if not isinstance(formula, Formula):
+            if not isinstance(formula, _Formula):
                 raise ValueError(f"'formula' value '{formula}' must be a Formula object")
             self._formulas.append(formula)
             return self
         if name:
             if not isinstance(name, str):
                 raise ValueError(f"'name' value '{name}' must be a string")
-            new_formula = Formula(**kwargs)
+            new_formula = _Formula(**kwargs)
             self._formulas.append(new_formula)
             return self
         raise ValueError("Either a 'name' for a new formula or an existing Formula 'formula' must be supplied")
@@ -620,7 +620,7 @@ class Chart:
     def __str__(self) -> str:
         return f"Chart(name='{self._name}')"
 
-    def __getitem__(self, key: str) -> Union[SeriesBase, Formula]:
+    def __getitem__(self, key: str) -> Union[SeriesBase, _Formula]:
         for series in self._series:
             if series._name == key:
                 return series
