@@ -14,11 +14,12 @@ from requests import HTTPError
 from requests import JSONDecodeError as RequestsJSONDecodeError
 
 from chalk._version import __version__ as chalkpy_version
-from chalk.client.client_protocol import ChalkAPIClientProtocol, OnlineQueryResponseProtocol
+from chalk.client import ChalkClient, OnlineQueryResponseProtocol
 from chalk.client.dataset import DatasetVersion, load_dataset
 from chalk.client.exc import (
     ChalkBaseException,
     ChalkComputeResolverException,
+    ChalkInvalidCredentialsException,
     ChalkOfflineQueryException,
     ChalkOnlineQueryException,
     ChalkResolverRunException,
@@ -59,6 +60,7 @@ T = TypeVar("T")
 
 
 class _ChalkClientConfig(BaseModel):
+    name: str
     client_id: str
     client_secret: str
     api_server: str
@@ -110,40 +112,42 @@ class OnlineQueryResponseImpl(OnlineQueryResponseProtocol):
         return v and v.value
 
 
-class ChalkAPIClientImpl(ChalkAPIClientProtocol):
+class ChalkAPIClientImpl(ChalkClient):
+    def __new__(cls, *args, **kwargs):
+        obj = object.__new__(ChalkAPIClientImpl)
+        obj.__init__(*args, **kwargs)
+        obj.__name__ = "ChalkClient"
+        obj.__qualname__ = "chalk.client.ChalkClient"
+        return obj
+
+    def __repr__(self):
+        return f"chalk.client.ChalkClient<{self._config.name}>"
+
     def __init__(
         self,
-        *,
-        client_id: Optional[str],
-        client_secret: Optional[str],
-        environment: Optional[str],
-        api_server: Optional[str],
+        client_id: Optional[str] = None,
+        client_secret: Optional[str] = None,
+        environment: Optional[str] = None,
+        api_server: Optional[str] = None,
+        _skip_cache: bool = False,
     ):
-        if client_id is not None and client_secret is not None:
-            self._config = _ChalkClientConfig(
-                client_id=client_id,
-                client_secret=client_secret,
-                api_server=api_server or "https://api.chalk.ai",
-                active_environment=environment,
-            )
-        else:
-            token = load_token()
-            if token is None:
-                raise ValueError(
-                    (
-                        "Could not find .chalk.yml config file for project, "
-                        "and explicit configuration was not provided. "
-                        "You may need to run `chalk login` from your command line, "
-                        "or check that your working directory is set to the root of "
-                        "your project."
-                    )
-                )
-            self._config = _ChalkClientConfig(
-                client_id=token.clientId,
-                client_secret=token.clientSecret,
-                api_server=api_server or token.apiServer or "https://api.chalk.ai",
-                active_environment=environment or token.activeEnvironment,
-            )
+        token = load_token(
+            client_id=client_id,
+            client_secret=client_secret,
+            active_environment=environment,
+            api_server=api_server,
+            skip_cache=_skip_cache,
+        )
+        if token is None:
+            raise ChalkInvalidCredentialsException()
+
+        self._config = _ChalkClientConfig(
+            name=token.name or "",
+            client_id=token.clientId,
+            client_secret=token.clientSecret,
+            api_server=token.apiServer or "https://api.chalk.ai",
+            active_environment=token.activeEnvironment,
+        )
 
         self._default_headers = {
             "Accept": "application/json",
