@@ -3,7 +3,7 @@ from __future__ import annotations
 import base64
 import collections.abc
 from datetime import date, datetime, time, timedelta
-from typing import Any, List, Optional, Type, cast
+from typing import Any, List, Optional, Type, Union, cast
 
 import cattrs
 import dateutil.parser
@@ -23,7 +23,7 @@ def unstructure_primitive_to_json(val: TPrimitive) -> TJSON:
     return _json_converter.unstructure(val)
 
 
-def structure_json_to_primitive(val: TJSON, typ: Type[TPrimitive]) -> TPrimitive:
+def structure_json_to_primitive(val: Union[TJSON, TPrimitive], typ: Type[TPrimitive]) -> TPrimitive:
     return _json_converter.structure(val, typ)
 
 
