@@ -2,16 +2,14 @@ from __future__ import annotations
 
 import copy
 from collections.abc import Iterable
-from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union
+from typing import Any, TypeVar, Union
 
+from chalk.features.feature_field import Feature
 from chalk.features.filter import Filter
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import get_name_with_duration
 from chalk.utils.collections import ensure_tuple
 
-if TYPE_CHECKING:
-    from chalk.features.feature_field import Feature
-
 T = TypeVar("T")
 
 __all__ = ["FeatureWrapper", "unwrap_feature"]
@@ -28,6 +26,9 @@ class FeatureWrapper:
         # Binding as a private variable as not to have naming conflicts user's use of __getattr__
         self._chalk_feature = feature
 
+    def __matmul__(self, other: int):
+        return FeatureWrapper(self._chalk_feature.for_version(other))
+
     def __hash__(self):
         return hash(self._chalk_feature)
 
@@ -69,7 +70,11 @@ class FeatureWrapper:
 
     def __or__(self, other: object):
         if other is None:
-            return Optional[self]
+            other = type(None)
+        if isinstance(other, type):
+            # The FeatureWrapper appears as a UnionType in a type annotation -- e.g.
+            # def my_resolver(name: User.name | None = None, ...)
+            return Union[other, self]  # type: ignore
         return self._cmp("or", other)
 
     def __repr__(self):
@@ -116,6 +121,7 @@ class FeatureWrapper:
         if len(self._chalk_feature.window_durations) > 0:
             return self._chalk_get_windowed_feature(*ensure_tuple(item))
 
+        assert isinstance(self._chalk_feature.typ.parsed_annotation, type)
         if self._chalk_feature.typ is not None and issubclass(self._chalk_feature.typ.parsed_annotation, DataFrame):
             f_copy = FeatureWrapper(copy.copy(self._chalk_feature))
 
@@ -127,7 +133,6 @@ class FeatureWrapper:
 
     def __getattr__(self, item: str):
         from chalk.features.feature_field import Feature
-        from chalk.features.feature_set import Features
 
         # Passing through __getattr__ on has_one features, as users can use getattr
         # notation in annotations for resolvers
@@ -135,8 +140,8 @@ class FeatureWrapper:
             # Short-circuiting on the dunders to be compatible with copy.copy
             raise AttributeError(item)
 
-        if self._chalk_feature.typ is not None and issubclass(self._chalk_feature.typ.underlying, Features):
-            for f in self._chalk_feature.typ.underlying.features:
+        if self._chalk_feature.joined_class is not None:
+            for f in self._chalk_feature.joined_class.features:
                 assert isinstance(f, Feature), f"HasOne feature {f} does not inherit from FeaturesBase"
                 if f.attribute_name == item:
                     return FeatureWrapper(self._chalk_feature.copy_with_path(f))
@@ -148,8 +153,6 @@ def unwrap_feature(maybe_feature_wrapper: Any) -> Feature:
 
     For example:
 
-    .. code-block::
-
         @features
         class FooBar:
             foo: str
@@ -158,8 +161,6 @@ def unwrap_feature(maybe_feature_wrapper: Any) -> Feature:
         type(FooBar.foo) is FeatureWrapper
         type(unwrap_feature(FooBar.foo)) is Feature
     """
-    from chalk.features.feature_field import Feature
-
     if isinstance(maybe_feature_wrapper, FeatureWrapper):
         maybe_feature_wrapper = maybe_feature_wrapper._chalk_feature
     if isinstance(maybe_feature_wrapper, Feature):
@@ -167,3 +168,13 @@ def unwrap_feature(maybe_feature_wrapper: Any) -> Feature:
     raise TypeError(
         f"{maybe_feature_wrapper} is of type {type(maybe_feature_wrapper).__name__}, expecting type FeatureWrapper"
     )
+
+
+def ensure_feature(feature: Union[str, Feature, FeatureWrapper, Any]) -> Feature:
+    if isinstance(feature, str):
+        return Feature.from_root_fqn(feature)
+    if isinstance(feature, FeatureWrapper):
+        return unwrap_feature(feature)
+    if isinstance(feature, Feature):
+        return feature
+    raise TypeError(f"Feature identifier {feature} of type {type(feature).__name__} is not supported.")
