@@ -12,6 +12,7 @@ from typing import (
     Dict,
     Generic,
     List,
+    Literal,
     Optional,
     Protocol,
     Sequence,
@@ -34,8 +35,9 @@ from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
 from chalk.features.filter import Filter
 from chalk.features.tag import Environments, Tags
 from chalk.serialization.parsed_annotation import ParsedAnnotation
+from chalk.sink import SinkIntegrationProtocol
 from chalk.state import StateWrapper
-from chalk.streams import StreamSource, Windowed, get_name_with_duration
+from chalk.streams import StreamSource, get_name_with_duration
 from chalk.streams.types import (
     StreamResolverParam,
     StreamResolverParamKeyedState,
@@ -45,15 +47,18 @@ from chalk.streams.types import (
 )
 from chalk.utils import MachineType
 from chalk.utils.annotation_parsing import ResolverAnnotationParser
-from chalk.utils.collection_type import GenericAlias
 from chalk.utils.collections import ensure_tuple
 from chalk.utils.duration import Duration, ScheduleOptions
 from chalk.utils.environment_parsing import env_var_bool
 
+try:
+    from types import UnionType
+except ImportError:
+    UnionType = Union
+
 if TYPE_CHECKING:
     from chalk.sql import BaseSQLSourceProtocol
 
-
 T = TypeVar("T")
 P = ParamSpec("P")
 
@@ -72,7 +77,7 @@ class StateDescriptor(Generic[T]):
 
 
 class FilterFunction(Protocol):
-    def __call__(self, *args: Feature) -> bool:
+    def __call__(self, *args: Union[Feature, Any]) -> bool:
         ...
 
 
@@ -83,10 +88,31 @@ class SampleFunction(Protocol):
 
 @dataclass
 class Cron:
+    """
+    Detailed options for specify the schedule and filtering
+    functions for Chalk batch jobs.
+    """
+
     schedule: Optional[ScheduleOptions] = None
+    """
+    The period of the cron job. Can be either a crontab (`"0 * * * *"`)
+    or a `Duration` (`"2h"`).
+    """
+
     filter: Optional[FilterFunction] = None
+    """Optionally, a function to filter down the arguments to consider.
+
+    See https://docs.chalk.ai/docs/resolver-cron#filtering-examples for more information.
+    """
+
     sample: Optional[SampleFunction] = None
+    """Explicitly provide the sample function for the cron job.
+
+    See https://docs.chalk.ai/docs/resolver-cron#custom-examples for more information.
+    """
+
     trigger_downstream: Optional[bool] = True
+    """Whether or not to trigger downstream resolvers"""
 
 
 def _flatten_features(output: Optional[Type[Features]]) -> Sequence[Feature]:
@@ -95,8 +121,7 @@ def _flatten_features(output: Optional[Type[Features]]) -> Sequence[Feature]:
     features = output.features
     if len(features) == 1 and isinstance(features[0], type) and issubclass(features[0], DataFrame):
         return features[0].columns
-    else:
-        return features
+    return features
 
 
 class Resolver(Generic[P, T]):
@@ -161,7 +186,6 @@ class SinkResolver(Resolver[P, T]):
         filename: str,
         doc: Optional[str],
         inputs: List[Feature],
-        output: Type[Features],
         fn: Callable[P, T],
         environment: Optional[List[str]],
         tags: Optional[List[str]],
@@ -171,14 +195,13 @@ class SinkResolver(Resolver[P, T]):
         max_delay: Optional[Duration],
         upsert: Optional[bool],
         owner: Optional[str],
-        integration: Optional[BaseSQLSourceProtocol] = None,
+        integration: Optional[Union[BaseSQLSourceProtocol, SinkIntegrationProtocol]] = None,
     ):
         self.owner = owner
         self.function_definition = function_definition
         self.fqn = fqn
         self.filename = filename
         self.inputs = inputs
-        self.output = output
         self.fn = fn
         self.__module__ = fn.__module__
         self.__doc__ = fn.__doc__
@@ -195,6 +218,7 @@ class SinkResolver(Resolver[P, T]):
         self.default_args = []
         self.max_staleness = None
         self.state = None
+        self.output = []
 
     def __repr__(self):
         return f"SinkResolver(name={self.fqn})"
@@ -224,6 +248,7 @@ class OnlineResolver(Resolver[P, T]):
         state: Optional[StateDescriptor],
         default_args: List[Optional[ResolverArgErrorHandler]],
         owner: Optional[str],
+        timeout: Optional[Duration],
     ):
         self.function_definition = function_definition
         self.fqn = fqn
@@ -244,6 +269,7 @@ class OnlineResolver(Resolver[P, T]):
         self.state = state
         self.default_args = default_args
         self.owner = owner
+        self.timeout = timeout
 
     def __repr__(self):
         return f"OnlineResolver(name={self.fqn})"
@@ -271,6 +297,7 @@ class OfflineResolver(Resolver[P, T]):
         when: Optional[Filter],
         default_args: List[Optional[ResolverArgErrorHandler]],
         owner: Optional[str],
+        timeout: Optional[Duration],
     ):
         self.when = when
         self.function_definition = function_definition
@@ -291,10 +318,10 @@ class OfflineResolver(Resolver[P, T]):
         self.state = state
         self.default_args = default_args
         self.owner = owner
+        self.timeout = timeout
 
-
-def __repr__(self):
-    return f"OfflineResolver(name={self.fqn})"
+    def __repr__(self):
+        return f"OfflineResolver(name={self.fqn})"
 
 
 @dataclasses.dataclass(frozen=True)
@@ -365,7 +392,7 @@ def parse_function(
 
     function_definition = inspect.getsource(fn)
     return_annotation = get_type_hints(fn).get("return")
-    if return_annotation is None:
+    if return_annotation is None and not ignore_return:
         raise TypeError(
             f"Resolver '{fqn}' must have a return annotation. See https://docs.chalk.ai/docs/resolver-outputs for more information."
         )
@@ -382,13 +409,13 @@ def parse_function(
             assert issubclass(return_annotation.typ.parsed_annotation, DataFrame)
             ret_val = Features[return_annotation.typ.parsed_annotation.columns]
         elif return_annotation.is_has_one:
-            assert issubclass(return_annotation.typ.underlying, Features)
-            ret_val = Features[return_annotation.typ.underlying.features]
+            assert return_annotation.joined_class is not None
+            ret_val = Features[return_annotation.joined_class.features]
         else:
             # function annotated like def get_account_id(user_id: User.id) -> User.account_id
             ret_val = Features[return_annotation]
 
-    if ret_val is None:
+    if ret_val is None and not ignore_return:
         if not isinstance(return_annotation, type):
             raise TypeError(f"return_annotation {return_annotation} of type {type(return_annotation)} is not a type")
         if issubclass(return_annotation, Features):
@@ -416,7 +443,7 @@ def parse_function(
         )
         arg = inputs[i]
 
-        if isinstance(arg, GenericAlias) and get_origin(arg) == Union:
+        if get_origin(arg) in (UnionType, Union):
             args = get_args(arg)
             if len(args) != 2:
                 raise bad_input()
@@ -437,13 +464,14 @@ def parse_function(
                 continue
             raise bad_input()
 
-        if isinstance(arg.typ, ParsedAnnotation) and isinstance(arg.typ.underlying, Windowed):
-            f = arg.typ.underlying
+        if isinstance(arg, Feature) and (arg.is_windowed or arg.typ.is_windowed):
+            # Windowed arguments in resolver signatures must specify a window bucket
+            available_windows = ", ".join(f"{x}s" for x in arg.window_durations)
             raise ValueError(
                 (
                     f"Resolver argument '{arg_name}' to '{fqn}' does not select a window. "
-                    f"Add a selected window, like {arg.name}('{next(iter(f._buckets), '')}'). "
-                    f"Available windows: {', '.join(f._buckets)}."
+                    f"Add a selected window, like {arg.name}('{next(iter(arg.window_durations), '')}'). "
+                    f"Available windows: {available_windows}."
                 )
             )
 
@@ -502,6 +530,18 @@ def parse_function(
     )
 
 
+class ResolverProtocol(Generic[T, P]):
+    __is_resolver__ = True
+    name: str
+
+    def __call__(self, args: P) -> T:
+        ...
+
+    @classmethod
+    def __is_resolver__(cls, o):
+        return hasattr(o, "__is_resolver__") and o.__is_resolver__ is True
+
+
 @overload
 def online(
     *,
@@ -511,7 +551,8 @@ def online(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
-) -> Callable[[Callable[P, T]], Callable[P, T]]:
+    timeout: Optional[Duration] = None,
+) -> Callable[[Callable[P, T]], ResolverProtocol[P, T]]:
     ...
 
 
@@ -519,7 +560,7 @@ def online(
 def online(
     fn: Callable[P, T],
     /,
-) -> Callable[P, T]:
+) -> ResolverProtocol[P, T]:
     ...
 
 
@@ -533,29 +574,30 @@ def online(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
-) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
-    """
-    :param fn: The function that you're decorating as a resolver.
-
-    :param environment: Environments are used to trigger behavior
+    timeout: Optional[Duration] = None,
+) -> Union[Callable[[Callable[P, T]], ResolverProtocol[P, T]], ResolverProtocol[P, T]]:
+    """Decorator to create an online resolver.
+    Parameters
+    ----------
+    environment
+        Environments are used to trigger behavior
         in different deployments such as staging, production, and
         local development. For example, you may wish to interact with
         a vendor via an API call in the production environment, and
         opt to return a constant value in a staging environment.
 
         Environment can take one of three types:
-
-            - None (default) - candidate to run in every environment
-            - str - run only in this environment
-            - list[str] - run in any of the specified environment and no others
+            - `None` (default) - candidate to run in every environment
+            - `str` - run only in this environment
+            - `list[str]` - run in any of the specified environment and no others
 
         Read more at https://docs.chalk.ai/docs/resolver-environments
-
-    :param owner: Owner allows you to specify an individual or team
-        who is responsible for this resolver. The Chalk Dashboard
-        will display this field, and alerts can be routed to owners.
-
-    :param tags: Tags allow you to scope requests within an
+    owner
+        Individual or team responsible for this resolver.
+        The Chalk Dashboard will display this field, and alerts
+        can be routed to owners.
+    tags
+        Allow you to scope requests within an
         environment. Both tags and environment need to match for a
         resolver to be a candidate to execute.
 
@@ -565,32 +607,61 @@ def online(
         staging environment.
 
         Read more at https://docs.chalk.ai/docs/resolver-tags
-
-    :param cron: You can schedule resolvers to run on a
-        pre-determined schedule via the cron argument to resolver
-        decorators.
+    cron
+        You can schedule resolvers to run on a pre-determined
+        schedule via the cron argument to resolver decorators.
 
         Cron can sample all examples, a subset of all examples,
         or a custom provided set of examples.
 
         Read more at https://docs.chalk.ai/docs/resolver-cron
+    timeout
+        You can specify the maximum duration to wait for the
+        resolver's result. Once the resolver's runtime exceeds
+        the specified duration, a timeout error will be returned
+        along with each output feature.
 
-    :param machine_type: You can optionally specify that resolvers
-        need to run on a machine other than the default. Must be
-        configured in your deployment.
+        Please use supported Chalk durations
+        'w', 'd', 'h', 'm', 's', and/or 'ms'.
 
-    :param when: Like tags, `when` can filter when a resolver
-        is eligible to run. Unlike tags, `when` can use feature values,
+        Read more at https://docs.chalk.ai/docs/timeout
+                 and https://docs.chalk.ai/docs/duration
+    when
+        Like tags, `when` can filter when a resolver is eligible
+        to run. Unlike tags, `when` can use feature values,
         so that you can write resolvers like:
 
-            @batch(when=User.risk_profile == "low" or User.is_employee)
-            def resolver_fn(...) -> ...:
-                ...
+        >>> @batch(when=User.risk_profile == "low" or User.is_employee)
+        ... def resolver_fn(...) -> ...:
+        ...     ...
 
-    :return: A callable function! You can unit test resolvers
-        as you would unit test any other code.
+    Other Parameters
+    ----------------
+    fn
+        The function that you're decorating as a resolver.
+    machine_type
+        You can optionally specify that resolvers need to run on a
+        machine other than the default. Must be configured in your
+        deployment.
 
-    Read more at https://docs.chalk.ai/docs/unit-tests
+    Returns
+    -------
+    Callable[[Any, ...], Any]
+        A callable function! You can unit-test resolvers as you would
+        unit-test any other code.
+
+        Read more at https://docs.chalk.ai/docs/unit-tests
+
+    Examples
+    --------
+    >>> @online
+    ... def name_match(
+    ...     name: User.full_name,
+    ...     account_name: User.bank_account.title
+    ... ) -> User.account_name_match_score:
+    ...     if name.lower() == account_name.lower():
+    ...         return 1.
+    ...     return 0.
     """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
@@ -621,6 +692,7 @@ def online(
             owner=owner,
             state=parsed.state,
             default_args=parsed.default_args,
+            timeout=timeout,
         )
 
         Resolver.registry.append(resolver)
@@ -643,7 +715,7 @@ def offline(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
-) -> Callable[[Callable[P, T]], Callable[P, T]]:
+) -> Callable[[Callable[P, T]], ResolverProtocol[P, T]]:
     ...
 
 
@@ -651,7 +723,7 @@ def offline(
 def offline(
     fn: Callable[P, T],
     /,
-) -> Callable[P, T]:
+) -> ResolverProtocol[P, T]:
     ...
 
 
@@ -665,29 +737,31 @@ def offline(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
-) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
-    """
-    :param fn: The function that you're decorating as a resolver.
+    timeout: Optional[Duration] = None,
+) -> Union[Callable[[Callable[P, T]], Callable[P, T]], ResolverProtocol[P, T]]:
+    """Decorator to create an offline resolver.
 
-    :param environment: Environments are used to trigger behavior
+    Parameters
+    ----------
+    environment
+        Environments are used to trigger behavior
         in different deployments such as staging, production, and
         local development. For example, you may wish to interact with
         a vendor via an API call in the production environment, and
         opt to return a constant value in a staging environment.
 
         Environment can take one of three types:
-
-            - None (default) - candidate to run in every environment
-            - str - run only in this environment
-            - list[str] - run in any of the specified environment and no others
+            - `None` (default) - candidate to run in every environment
+            - `str` - run only in this environment
+            - `list[str]` - run in any of the specified environment and no others
 
         Read more at https://docs.chalk.ai/docs/resolver-environments
-
-    :param owner: Owner allows you to specify an individual or team
+    owner
+        Allows you to specify an individual or team
         who is responsible for this resolver. The Chalk Dashboard
         will display this field, and alerts can be routed to owners.
-
-    :param tags: Tags allow you to scope requests within an
+    tags
+        Allow you to scope requests within an
         environment. Both tags and environment need to match for a
         resolver to be a candidate to execute.
 
@@ -697,32 +771,48 @@ def offline(
         staging environment.
 
         Read more at https://docs.chalk.ai/docs/resolver-tags
-
-    :param cron: You can schedule resolvers to run on a
-        pre-determined schedule via the cron argument to resolver
-        decorators.
+    cron
+        You can schedule resolvers to run on a pre-determined
+        schedule via the cron argument to resolver decorators.
 
         Cron can sample all examples, a subset of all examples,
         or a custom provided set of examples.
 
         Read more at https://docs.chalk.ai/docs/resolver-cron
+    timeout
+        You can specify the maximum duration to wait for the
+        resolver's result. Once the resolver's runtime exceeds
+        the specified duration, a timeout error will be raised.
 
-    :param machine_type: You can optionally specify that resolvers
-        need to run on a machine other than the default. Must be
-        configured in your deployment.
+        Please use supported Chalk durations
+        'w', 'd', 'h', 'm', 's', and/or 'ms'.
 
-    :param when: Like tags, `when` can filter when a resolver
+        Read more at https://docs.chalk.ai/docs/timeout
+                 and https://docs.chalk.ai/docs/duration
+    when
+        Like tags, `when` can filter when a resolver
         is eligible to run. Unlike tags, `when` can use feature values,
-        so that you can write resolvers like:
+        so that you can write resolvers like::
 
-            @batch(when=User.risk_profile == "low" or User.is_employee)
-            def resolver_fn(...) -> ...:
-                ...
+        >>> @batch(when=User.risk_profile == "low" or User.is_employee)
+        ... def resolver_fn(...) -> ...:
+        ...    ...
 
-    :return: A callable function! You can unit test resolvers
+    Other Parameters
+    ----------------
+    fn
+        The function that you're decorating as a resolver.
+    machine_type
+        You can optionally specify that resolvers need to run on
+        a machine other than the default. Must be configured in
+        your deployment.
+
+    Returns
+    -------
+    Callable[[Any, ...], Any]
+        A callable function! You can unit-test resolvers
         as you would unit test any other code.
-
-    Read more at https://docs.chalk.ai/docs/unit-tests
+        Read more at https://docs.chalk.ai/docs/unit-tests
     """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
@@ -752,6 +842,7 @@ def offline(
             when=when,
             owner=owner,
             default_args=parsed.default_args,
+            timeout=timeout,
         )
         Resolver.registry.append(resolver)
         if Resolver.hook:
@@ -771,7 +862,7 @@ def sink(
     debounce: Optional[Duration] = None,
     max_delay: Optional[Duration] = None,
     upsert: Optional[bool] = None,
-    integration: Optional[BaseSQLSourceProtocol] = None,
+    integration: Optional[Union[BaseSQLSourceProtocol, SinkIntegrationProtocol]] = None,
 ) -> Callable[[Callable[P, T]], Callable[P, T]]:
     ...
 
@@ -796,7 +887,74 @@ def sink(
     max_delay: Optional[Duration] = None,
     upsert: Optional[bool] = None,
     integration: Optional[BaseSQLSourceProtocol] = None,
+    owner: Optional[str] = None,
 ) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
+    """Decorator to create a stream resolver.
+
+    Parameters
+    ----------
+    environment
+        Environments are used to trigger behavior
+        in different deployments such as staging, production, and
+        local development. For example, you may wish to interact with
+        a vendor via an API call in the production environment, and
+        opt to return a constant value in a staging environment.
+
+        Environment can take one of three types:
+            - `None` (default) - candidate to run in every environment
+            - `str` - run only in this environment
+            - `list[str]` - run in any of the specified environment and no others
+
+        Read more at https://docs.chalk.ai/docs/resolver-environments
+    tags
+        Allow you to scope requests within an
+        environment. Both tags and environment need to match for a
+        resolver to be a candidate to execute.
+
+        You might consider using tags, for example, to change out
+        whether you want to use a sandbox environment for a vendor,
+        or to bypass the vendor and return constant values in a
+        staging environment.
+
+        Read more at https://docs.chalk.ai/docs/resolver-tags
+    buffer_size
+        Count of updates to buffer.
+    owner
+        The individual or team responsible for this resolver.
+        The Chalk Dashboard will display this field, and alerts
+        can be routed to owners.
+
+    Other Parameters
+    ----------------
+    fn
+        The function that you're decorating as a resolver.
+    machine_type
+        You can optionally specify that resolvers need to run on a
+        machine other than the default. Must be configured in your
+        deployment.
+    debounce
+    max_delay
+    upsert
+    integration
+
+    Examples
+    --------
+    >>> @sink
+    ... def process_updates(
+    ...     uid: User.id,
+    ...     email: User.email,
+    ...     phone: User.phone,
+    ... ):
+    ...     user_service.update(uid=uid, email=email, phone=phone)
+
+
+    Returns
+    -------
+    Callable[[Any, ...], Any]
+        A callable function! You can unit-test sinks as you
+        would unit test any other code.
+        Read more at https://docs.chalk.ai/docs/unit-tests
+    """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
     caller_globals = caller_frame.frame.f_globals
@@ -804,14 +962,12 @@ def sink(
 
     def decorator(fn: Callable[P, T], cf: str = caller_filename):
         parsed = parse_function(fn, caller_globals, caller_locals, ignore_return=True)
-        assert parsed.output is not None
         resolver = SinkResolver(
             filename=cf,
             function_definition=parsed.function_definition,
             fqn=parsed.fqn,
             doc=parsed.doc,
             inputs=parsed.inputs,
-            output=parsed.output,
             fn=fn,
             environment=None if environment is None else list(ensure_tuple(environment)),
             tags=None if tags is None else list(ensure_tuple(tags)),
@@ -821,6 +977,7 @@ def sink(
             max_delay=max_delay,
             upsert=upsert,
             integration=integration,
+            owner=owner,
         )
         SinkResolver.registry.append(resolver)
         return resolver
@@ -833,38 +990,44 @@ class StreamResolver(Resolver[P, T]):
 
     def __init__(
         self,
-        caller_globals: Optional[Dict[str, Any]],
-        caller_locals: Optional[Dict[str, Any]],
-        caller_filename: str,
+        function_definition: str,
+        fqn: str,
+        filename: str,
         source: StreamSource,
         fn: Callable[P, T],
-        mode: Optional[str],
-        signature: StreamResolverSignature,
         environment: Optional[List[str]],
+        doc: Optional[str],
+        mode: Optional[Literal["continuous", "tumbling"]],
+        module: str,
         machine_type: Optional[str],
-        output: Type[Features],
         message: Optional[Type[Any]],
+        output: Type[Features],
+        signature: StreamResolverSignature,
+        state: Optional[StateDescriptor],
+        sql_query: Optional[str],
+        owner: Optional[str],
     ):
-        parsed = parse_function(fn, caller_globals, caller_locals, allow_custom_args=True, is_streaming_resolver=True)
-        self.function_definition = parsed.function_definition
-        self.fqn = parsed.fqn
-        self.filename = caller_filename
+        self.function_definition = function_definition
+        self.fqn = fqn
+        self.filename = filename
+        self.inputs = []
+        self.doc = doc
         self.source = source
-        self.mode = mode
         self.fn = fn
         self.environment = environment
-        self.doc = parsed.doc
-        self.inputs = []
-        self.__doc__ = parsed.doc
-        self.__module__ = fn.__module__
+        self.__doc__ = doc
+        self.__module__ = module
         self.machine_type = machine_type
         self.max_staleness = None
-        self.state = parsed.state
-        self.tags = None  # TODO -- should there be tags?
-        self.signature = signature
-        self.output = output
         self.message = message
+        self.output = output
+        self.mode = mode
+        self.signature = signature
+        self.state = state
+        self.sql_query = sql_query
+        self.tags = None
         self.default_args = []
+        self.owner = owner
         fqn_to_windows = {o.fqn: o.window_durations for o in _flatten_features(self.output) if o.is_windowed}
         if len(set(tuple(v) for v in fqn_to_windows.values())) > 1:
             fqn_to_declared_windows = {
@@ -979,7 +1142,7 @@ def _parse_stream_resolver_param(
     )
 
 
-def parse_stream_resolver_params(
+def _parse_stream_resolver_params(
     user_func: Callable,
     *,
     resolver_fqn_for_errors: str,
@@ -1011,7 +1174,7 @@ def parse_stream_resolver_params(
     return params
 
 
-def parse_stream_resolver_output_features(
+def _parse_stream_resolver_output_features(
     user_func: Callable,
     *,
     resolver_fqn_for_errors: str,
@@ -1055,10 +1218,12 @@ def parse_and_register_stream_resolver(
     environment: Optional[Union[List[str], str]] = None,
     machine_type: Optional[MachineType] = None,
     message: Optional[Type[Any]] = None,
+    sql_query: Optional[str] = None,
+    owner: Optional[str] = None,
 ) -> StreamResolver[P, T]:
     fqn = f"{fn.__module__}.{fn.__name__}"
     annotation_parser = ResolverAnnotationParser(fn, caller_globals, caller_locals)
-    output_features = parse_stream_resolver_output_features(
+    output_features = _parse_stream_resolver_output_features(
         fn,
         resolver_fqn_for_errors=fqn,
     )
@@ -1070,7 +1235,7 @@ def parse_and_register_stream_resolver(
         else output_features.features
     )
     is_windowed_resolver = any(x.is_windowed for x in flattened_output_features)
-    params = parse_stream_resolver_params(
+    params = _parse_stream_resolver_params(
         fn,
         resolver_fqn_for_errors=fqn,
         annotation_parser=annotation_parser,
@@ -1086,19 +1251,25 @@ def parse_and_register_stream_resolver(
         params=params,
         output_feature_fqns=output_feature_fqns,
     )
+    parsed = parse_function(fn, caller_globals, caller_locals, allow_custom_args=True, is_streaming_resolver=True)
 
     resolver = StreamResolver(
-        caller_globals=caller_globals,
-        caller_locals=caller_locals,
-        caller_filename=caller_filename,
+        function_definition=parsed.function_definition,
+        fqn=parsed.fqn,
+        filename=caller_filename,
         source=source,
-        mode=mode,
         fn=fn,
         environment=None if environment is None else list(ensure_tuple(environment)),
+        doc=parsed.doc,
+        module=fn.__module__,
+        mode=mode,
         machine_type=machine_type,
-        signature=signature,
-        output=output_features,
         message=message,
+        output=output_features,
+        signature=signature,
+        state=parsed.state,
+        sql_query=None,
+        owner=owner,
     )
 
     StreamResolver.registry.append(resolver)
