@@ -8,6 +8,9 @@ class UpsertFeatureIdGQL:
     fqn: str
     name: str
     namespace: str
+    className: Optional[str] = None
+    attributeName: Optional[str] = None
+    explicitNamespace: Optional[bool] = None
 
 
 @dataclass
