@@ -3,6 +3,7 @@ from __future__ import annotations
 import contextlib
 import functools
 import inspect
+import json
 import os
 import os.path
 import warnings
@@ -28,6 +29,7 @@ from typing import (
 import pyarrow as pa
 
 from chalk.features import Feature, FeatureConverter, Features, FeatureWrapper, unwrap_feature
+from chalk.integrations.named import load_integration_variable
 from chalk.sql._internal.chalk_query import ChalkQuery
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.sql._internal.string_chalk_query import StringChalkQuery
@@ -294,6 +296,19 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         query_string = compiled_query.string
         return query_string, compiled_query.positiontup or [], compiled_query.params
 
+    def load_env_engine_args(self, name: str) -> Mapping[str, Any]:
+        """
+        Loads additional engine arguments from env var "{name}_ENGINE_ARGUMENTS"
+        """
+
+        extra_args = load_integration_variable(integration_name=name, name="ENGINE_ARGUMENTS")
+        if extra_args is None:
+            return {}
+        else:
+            extra_args = json.loads(extra_args)
+            assert isinstance(extra_args, dict), "ENGINE_ARGUMENTS must be a JSON object"
+            return extra_args
+
     @functools.lru_cache(None)  # pyright: ignore[reportIncompatibleMethodOverride]
     def get_engine(self) -> Engine:
         from sqlalchemy.engine import create_engine
