@@ -5,12 +5,11 @@ import typing
 import warnings
 from typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union
 
-from typing_extensions import Annotated, get_args, get_origin
+from typing_extensions import Annotated, TypeAlias, get_args, get_origin
 
 from chalk.features import Feature, Features, FeatureSetBase
 from chalk.importer import get_directory_root, import_all_python_files_from_dir
 from chalk.streams import Windowed
-from chalk.utils.collection_type import GenericAlias
 
 try:
     from types import UnionType
@@ -39,14 +38,18 @@ class StubGenerator:
     ) -> str:
         """Add an import for item.
 
-        If ``item`` is already imported, then it will not be imported again.
+        If `item` is already imported, then it will not be imported again.
         Its existing import name will be returned
 
-        Args:
-            item: The thing to import. Should be the full path to the item (e.g. ``typing.Optional``)
+        Parameters
+        ----------
+        item
+            The thing to import. Should be the full path to the item (e.g. `typing.Optional`)
 
-        Returns:
-            str: The name that ``item`` will be imported as.
+        Returns
+        -------
+        str
+            The name that `item` will be imported as.
         """
         if isinstance(item, typing._SpecialForm):
             item = str(item)
@@ -80,16 +83,18 @@ class StubGenerator:
         item = f"__stubgen_{'_'.join(item_split)}"
         return item
 
-    def _register_imports_for_annotation(self, annotation: Union[type, Windowed, GenericAlias]) -> str:
-
+    def _register_imports_for_annotation(self, annotation: Union[type, Windowed]) -> str:
         if annotation == type(None):
             return "None"
+        if annotation is ...:
+            return "..."
         origin, args = get_origin(annotation), get_args(annotation)
         if origin is None:
             assert args == ()
             # Likely a "simple" annotation, like an enum class
             if isinstance(annotation, Windowed):
                 origin_name = self._add_import(annotation, import_as="Windowed")
+                assert annotation._kind is not None
                 arg_name = self._register_imports_for_annotation(annotation._kind)
                 return f"{origin_name}[{arg_name}]"
             elif annotation.__module__ == "builtins":
@@ -319,6 +324,13 @@ def configure_stubgen_argparse(parser: argparse.ArgumentParser):
         default=None,
         help="Folder containing custom type stubs. By default, will use the `typings` folder, relative to the project root",
     )
+    parser.add_argument(
+        "--verbose",
+        "-v",
+        action="store_true",
+        default=False,
+        help="Whether to print verbose error messages when stubgen fail",
+    )
 
 
 def run_stubgen(args: argparse.Namespace):
@@ -338,9 +350,15 @@ def run_stubgen(args: argparse.Namespace):
         warnings.warn(
             f"Stubs may be incomplete due to errors in loading the following files: {', '.join([x.filename for x in failed_imports])}"
         )
+        if args.verbose:
+            for x in failed_imports:
+                print(f"Failed import: {x.filename}: {x.traceback}")
     stubgen = StubGenerator()
     for features_cls in FeatureSetBase.registry.values():
-        stubgen.register_features_class(features_cls)
+        try:
+            stubgen.register_features_class(features_cls)
+        except Exception as e:
+            raise RuntimeError(f"Error processing features class {features_cls}") from e
     lines = stubgen.generate_features_decorator_stub_file()
     folder = os.path.join(stub_path, "chalk", "features")
     os.makedirs(folder, exist_ok=True)
